#include <sys/types.h> // open()
#include <sys/stat.h> // open()
#include <fcntl.h> // open()
#include <string.h> // strerror()
#include <errno.h> // strerror()
#include <unistd.h> // read()
#include <sys/ioctl.h> // ioctl()
#include <sapi.h>
#include <dsp.h>


#define READ_SIZE (160)


typedef struct {
	void *payload;
	size_t payload_size;
} payload_t;


static dsp_read_t __read_cb = NULL;
pthread_t __dsp_reader;


static inline payload_t *payload_alloc()
{
	return((payload_t *)sapi_malloc(sizeof(payload_t)));
}


static inline void payload_free(payload_t *payload)
{
	if(!payload)
		return;
	
	sapi_free(payload->payload);
	sapi_free(payload);
}


/**
 * Read from any DSP source and send to peer. One thread for each peer.
 */
static void *dsp_read_thread(void *arg)
{
	int fd = open("/tmp/test.wav", O_RDONLY), n;
	u8_t buf[2*READ_SIZE];

	struct timespec req;

	req.tv_sec  = 0;
	req.tv_nsec = 10*1000000;

	if(fd < 0) {
		sapi_logf("/tmp/test.wav: %s", strerror(errno));
		return(NULL);
	}

	while(__read_cb) {

		n = read(fd, buf, sizeof(buf));
		if(n != sizeof(buf)) {
			sapi_logf("could only read %u bytes", n);
		}

		if(n > 0) {
			__read_cb((void *)buf, READ_SIZE, arg);
			//			usleep(1);
		} else {
			__read_cb((void *)buf, 0, arg); /* go to idle */
			__read_cb = NULL;
			//			(void)lseek(fd, 0, SEEK_SET);
		}
		nanosleep(&req, NULL);
	}

	DSP_DEBUG_PRINTF("dsp_read_thread(): exit");

	if(fd != -1)
		close(fd);

	return(NULL);
}


void dsp_thread_start(dsp_read_t read, void *read_param)
{
	__read_cb    = read;

	(void)pthread_create(&__dsp_reader, NULL, dsp_read_thread, read_param);

	sapi_logf("new DSP thread with ID 0x%08x", __dsp_reader);
}


void dsp_thread_stop(void)
{
	sapi_logf("stopping DSP thread with ID 0x%08x", __dsp_reader);
	__read_cb = NULL;
}
