#include <unistd.h> // for write()
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include <strans.h>
#include <rtp.h>
#include <sdp.h>
#include <g711.h>
#include "dsp_thread.h"
#include "ua.h"

/** @todo (alfredh) this is ugly */
static sdp_t *this_sdp = NULL;
static ua_t *ua = NULL;


/**
 * Handler for DSP events.
 */
static void dsp_recv(void *payload, size_t payload_size, void *arg)
{
	u8_t encoded_buf[payload_size];
	u16_t i;
	s16_t *pcm = (s16_t *)payload;
	sdp_t *sdp = (sdp_t *)arg;
	sdp_attrib_t *attrib = NULL;

	/* terminate session */
	if(payload_size == 0) {
		sdp_free(this_sdp);
		this_sdp = NULL;
		return;
	}

	if(!sdp)
		return;

	for(i=0;i<payload_size;i++)
		encoded_buf[i] = pcm2ulaw(*pcm++); /** @todo hardcoded */

	attrib = (sdp_attrib_t *)sapi_list_get_head(sdp->media->attribs);
	if(attrib) {
	rtp_send(sdp->host, sdp->media->port, encoded_buf, payload_size,
		 attrib->rtp_pt);
	}
}


/**
 * Error handler for Server Transactions.
 */
static void s_err(const u8_t *reason, void *arg)
{
	sapi_logf("Server Transaction Error : %s", reason);
}


#if 0
/**
 * Scan media list and remove unsupported types
 */
static ret_t media_filter_list(void *value, u32_t n, void *arg1, void *arg2)
{
	const sdp_attrib_t *attrib = (const sdp_attrib_t *)value;
	u8_t i;

	(void)n;
	(void)arg2;

#define NO_CODECS (2)

	sapi_logf("media filter %s node %lu", attrib->rtpmap, n);

	for(i=0;i<NO_CODECS;i++) {

		/* if matching static, continue scanning the list */
		if(attrib->rtp_pt == 0 || attrib->rtp_pt == 8) {
			break;
		}

		/* if matching dynamic, continue scanning the list */
		if(sapi_strcmp_ci(attrib->rtpmap, "PCMU")
		   ||
		   sapi_strcmp_ci(attrib->rtpmap, "PCMA")) {
			break;
		}
	}

	/* if not found, remove from list */
	if(i == NO_CODECS) {
		node_t *node = ((list_t *)arg1)->start;

		while(node) {
			if(node->value == value) {
				sapi_logf("removing codec %s", attrib->rtpmap);
				sapi_list_remove((list_t *)arg1, (void *)node, attrib_free);
				return(OK);
			}
			node = node->next;
		}
	}
	return(OK);
}
#endif


static void handle_message(const msg_t *req, const u8_t *hkey)
{
	msg_t *resp;
	u8_t message_cmd[] = "/home/alfredh/cvsroot/other/ec/perl/message.pl";
	u8_t *msg_cmd = sapi_sprintf("%s \"%s\"", message_cmd, req->body);

	sapi_logf("incoming message from '%s'", req->request->method);
	sapi_logf("body: %s", req->body);

	resp = msg_response(req, "200", "OK");
	strans_resp(hkey, resp);

	/* syncronous call */
	if(msg_cmd)
		system(msg_cmd);
	else
		sapi_logf("sorry errors");

	sapi_free(msg_cmd);
}


/**
 * Request handler for Server Transactions.
 */
static void s_request(const msg_t *req, const msg_t *oreq, const u8_t *hkey, const u8_t *ohkey, void *arg)
{
	msg_t *resp;
	u8_t *from_str;
	static int busy = 0;

	if(busy)
		return;
	busy = 1;

	from_str = msg_address_display_str(req->from);
	sapi_logf("incoming request '%s' from '%s'", req->request->method, from_str);
	sapi_free(from_str);

	if(sapi_strcmp(req->request->method, "INVITE")) {

		if(this_sdp) {
			resp = msg_response(req, "486", "Busy Here");
			strans_resp(hkey, resp);
			msg_deref(resp);
			goto out;
		}

		this_sdp = sdp_parse(req->body);
		if(this_sdp) {
			/* filter out useless codecs */
			list_t *attribs = this_sdp->media->attribs;
			//			(void)(sapi_list_apply(attribs, media_filter_list, attribs, NULL));

			if(sapi_list_nnodes(attribs) > 0) {

				u8_t *body;
				sdp_t *sdp_echo = sdp_create("0.0.0.0", "audio", 5004, "RTP/AVP");
				if(!sdp_echo) {
					sapi_logf("could not create SDP echo");
					goto out;
				}
				sdp_media_add(sdp_echo, 0 /* ulaw */, "PCMU", 8000, NULL);

				sapi_logf("SDP ok host'%s:%u'", this_sdp->host, this_sdp->media->port);

				body = sdp_str(sdp_echo);
				if(!body)
					sapi_logf("sdp_str() error");
				sdp_free(sdp_echo);

				resp = msg_response(req, "200", "OK");

				/* echo back SDP payload */
				msg_add_header(resp, "Content-Type", "application/sdp");
				msg_add_header_u32(resp, "Content-Length", sapi_strlen(body));
				msg_set_body(resp, body);
				sapi_free(body);

				strans_resp(hkey, resp);
				dsp_thread_start(dsp_recv, (void *)this_sdp);
				goto out;
			}
		}

		/* not good */
		resp = msg_response(req, "488", "Not Acceptable Here");
		strans_resp(hkey, resp);

		msg_deref(resp);
	}
	else if(sapi_strcmp(req->request->method, "BYE")){

		if(this_sdp) {
			dsp_thread_stop();
			resp = msg_response(req, "200", "OK");
			strans_resp(hkey, resp);
			sdp_free(this_sdp);
			this_sdp = NULL;
		} else {
			resp = msg_response(req, "481", "Call/Transaction Does Not Exist");
			strans_resp(hkey, resp);
		}
	}
	else if(sapi_strcmp(req->request->method, "MESSAGE")){
		handle_message(req, hkey);
	}
	else {
		sapi_logf("unhandled request %s", req->request->method);
		
		resp = msg_response(req, "501", "Not Implemented");
		strans_resp(hkey, resp);
		msg_deref(resp);
	}
 out:
	busy = 0;
}


/**
 * Transport layer handler for incoming packets.
 */
static void tp_recv(const tp_t *tp, const msg_t *msg, void *param)
{
	ua_t *ua = (ua_t *)param;

	if(msg->response) {		
		ctrans_resp(msg);
	}
	else if(msg->request) {
		strans_req(tp, msg, s_request, s_err, ua);
	}
}


static void autoregister(void *arg, u8_t *sarg)
{
	ua_t *ua = (ua_t *)arg;

	ua_register(ua, 360);
	sapi_oneshot(3000, autoregister, ua, NULL);
}


/**
 * Signal handler for signal SIGTERM.
 */
static void dying_cb(int sig)
{
	sapi_logf("Homocide attempt with signal %d", sig);

	if(ua) {
		sapi_logf("Unregistering...");
		ua_register(ua, 0);
		sleep(1);
		ua_free(ua);
		ua = NULL;
	}
	exit(0);
}


/**
 * Main program.
 */
int main(int argc, char **argv)
{
	struct sigaction sa;

	/* install signal handler */
	sa.sa_handler = dying_cb;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	(void)sigaction(SIGTERM, &sa, NULL);

	//	sapi_daemon();
	sapi_log_init("/var/log/aap.log");
	sapi_oneshot_init();

	ua = ua_init();
	if(!ua)
		exit(-1);
	
	tp_init(ua->local->url->port, tp_recv, ua);
	ctrans_init();
	strans_init();

	rtp_init(0, NULL, ua);

	autoregister(ua, NULL);
	
	sapi_logf("AAP Ready with pid %u", getpid());

	for(;;) {
		sleep(2000);
	}

	dying_cb(SIGTERM);

	return(0);
}
