#include <stdlib.h> // for rand()
#include <sys/types.h> // for open()
#include <sys/stat.h> // for open()
#include <fcntl.h> // for open()
#include <string.h> // for strerror()
#include <errno.h> // for strerror()
#include <unistd.h>
#include <time.h>
#include <version.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include "ua.h"
#include <ctype.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netdb.h>              /* gethostbyname, getnetbyname */
#include <net/ethernet.h>       /* struct ether_addr */
#include <linux/wireless.h>


#define USERNAME "aap"
#define DOMAINNAME "sxdesign.com"


ua_t *ua_init(void)
{
	ua_t *ua = (ua_t *)sapi_malloc((u32_t)sizeof(ua_t));
	u8_t hostname[255] = "10.47.12.101";

	ua->str_local  = sapi_sprintf("sip:%s@%s:%u",
				      USERNAME, hostname, 5062);
	ua->str_global = sapi_sprintf("sip:%s@%s", USERNAME, DOMAINNAME);
	ua->str_proxy  = sapi_sprintf("sip:%s", DOMAINNAME);

	ua->local  = msg_address_alloc();
	ua->global = msg_address_alloc();
	ua->proxy  = msg_url_alloc();

	if(msg_address_parse(ua->local, ua->str_local) != OK) {
		sapi_logf("Error parsing local address: '%s'", ua->str_local);
		goto out_err;
	}

	if(msg_address_parse(ua->global, ua->str_global) != OK) {
		sapi_logf("Error parsing global address: '%s'", ua->str_global);
		goto out_err;
	}

	if(msg_url_parse(ua->proxy, ua->str_proxy) != OK) {
		sapi_logf("Error parsing proxy URI: '%s'", ua->str_proxy);
		goto out_err;
	}	

	sapi_logf("SIP UA Version: " SIPD_VERSION);
	sapi_logf("Local Address : '%s'", ua->str_local);
	sapi_logf("Global Address: '%s'", ua->str_global);
	sapi_logf("Proxy URI     : '%s'", ua->str_proxy);

	return(ua);

out_err:
	ua_free(ua);
	return(NULL);
}


void ua_free(ua_t *ua)
{
	if(!ua)
		return;

	sapi_free(ua->str_local);
	sapi_free(ua->str_global);
	sapi_free(ua->str_proxy);

	msg_address_free(ua->local);
	msg_address_free(ua->global);
	msg_url_free(ua->proxy);

	msg_address_free(ua->last);

	msg_deref(ua->inv);

	sapi_free(ua);
}


static void err(const msg_t *req, const u8_t *reason, void *arg, u8_t *sarg)
{
	sapi_logf("%s Error: %s", req->request->method, reason);
}

static void resp(const msg_t *req, const msg_t *resp, void *arg, u8_t *sarg)
{
	sapi_logf("%s Response: %s (%s)",
		  req->request->method,
		  resp->response->statuscode,
		  resp->response->reasonphrase);
}


char *get_ssid(void)
{
	struct iwreq wrq;
	char *essid = malloc(IW_ESSID_MAX_SIZE + 1);
	int ret;

	static const int families[] = {
		AF_INET, AF_IPX, AF_AX25, AF_APPLETALK
	};
	unsigned int  i;
	int  sock;
        
	/* Try all families we support */
        for(i = 0; i < sizeof(families)/sizeof(int); ++i) {
		/* Try to open the socket, if success returns it */
		sock = socket(families[i], SOCK_DGRAM, 0);
		if(sock >= 0)
		        break;
	}

	wrq.u.essid.pointer = (caddr_t) essid;
	wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;

	strncpy(wrq.ifr_name, "eth1", IFNAMSIZ);

	ret = ioctl(sock, SIOCGIWESSID, &wrq);
	if(ret < 0 || strlen(essid) <= 0) {
		free(essid);
		essid = NULL;
	}

	close(sock);
	return(essid);
}


void ua_register(ua_t *ua, s32_t expires)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(ua->local->url->host, ua->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), ua->global->url->host);
	tp_t *tp        = tp_alloc();
	static u32_t cseq;
	address_t *contact;
	char *ssid = get_ssid();
	
	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	msg_via_addparam(via, "branch", branch);

	contact = msg_address_dup(ua->local);
	msg_address_addparam_u32(contact, "expires", expires);

	msg_set_request(msg, "REGISTER", ua->proxy);
	msg_add_via(msg, via);
	msg_set_to(msg, ua->global);
	msg_set_from(msg, ua->global);
	msg_add_contact(msg, contact);
	msg_set_call_id(msg, call_id);
	msg_set_cseq(msg, cseq++, "REGISTER");
	msg_set_user_agent(msg, USER_AGENT);
	if(ssid) {
		sapi_logf("ssid '%s'\n", ssid);
		msg_add_header(msg, "X-SSID", ssid);
		free(ssid);
	}
	
	ctrans_req(tp, msg, resp, err, NULL, NULL);

	msg_address_free(contact);
	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}
