#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include "ct.h"


ct_t *ct_alloc(void)
{
	ct_t *ct   = (ct_t *)sapi_malloc((u32_t)sizeof(ct_t));

	ct->nrefs = 1;
	sapi_mutex_init(&ct->mutex);

	return(ct);
}


static void ct_free(ct_t *ct)
{
	if(!ct) {
		CTRANS_DEBUG_PRINTF("ct_free(): tried to free NULL pointer");
		return;
	}

	sapi_free(ct->hkey);
	tp_deref(ct->tp);
	msg_deref(ct->req);
	msg_deref(ct->ack);
	sapi_free(ct->sarg);

	sapi_free(ct);
}


ct_t *ct_ref(ct_t *ct)
{
	if(!ct)
		return(NULL);

	sapi_mutex_lock(&ct->mutex);
	ct->nrefs++;
	sapi_mutex_unlock(&ct->mutex);

	return(ct);
}


void ct_deref(void *arg)
{
	ct_t *ct = (ct_t *)arg;
	u32_t nrefs;

	if(!ct)
		return;

	sapi_mutex_lock(&ct->mutex);
	nrefs = --ct->nrefs;
	sapi_mutex_unlock(&ct->mutex);

	if(!nrefs)
		ct_free(ct);
}


void ct_set_state(ct_t *ct, state_t state)
{
	if(!ct)
		return;

	sapi_mutex_lock(&ct->mutex);
	ct->state = state;
	sapi_mutex_unlock(&ct->mutex);
}


state_t ct_get_state(ct_t *ct)
{
	state_t state;

	if(!ct)
		return(INVALID);

	sapi_mutex_lock(&ct->mutex);
	state = ct->state;
	sapi_mutex_unlock(&ct->mutex);

	return(state);
}


void ct_inc_ntx(ct_t *ct)
{
	if(!ct)
		return;

	sapi_mutex_lock(&ct->mutex);
	ct->ntx++;
	sapi_mutex_unlock(&ct->mutex);
}


u32_t ct_get_ntx(ct_t *ct)
{
	u32_t ntx;

	if(!ct)
		return(0);

	sapi_mutex_lock(&ct->mutex);
	ntx = ct->ntx;
	sapi_mutex_unlock(&ct->mutex);

	return(ntx);
}
