#include <sapi.h>
#include <msg.h>
#include "dialog.h"


static dialog_t *dialog_alloc()
{
	return((dialog_t *)sapi_malloc((u32_t)sizeof(dialog_t)));
}


static dialog_t *dialog_find(list_t *dialogs, u8_t *call_id, u8_t *ltag, u8_t *rtag)
{
	dialog_t *d;
	node_t *node;

	node = dialogs->start;

	while(node) {
		d = (dialog_t *)node->value;
		node = node->next;

		if(sapi_strcmp(d->call_id, call_id) == FALSE)
			continue;

		if(sapi_strcmp(d->ltag, ltag) == FALSE)
			continue;

		if(sapi_strcmp(d->rtag, rtag) == FALSE)
			continue;

		return(d);
	}

	return(NULL);
}


void dialog_free(void *ptr)
{
	dialog_t *dialog = (dialog_t *)ptr;

	sapi_free(dialog->call_id);
	sapi_free(dialog->ltag);
	sapi_free(dialog->rtag);
	sapi_free(dialog->luri);
	sapi_free(dialog->ruri);
	sapi_free(dialog);
}


void dialog_create_outbound(list_t *dialogs, msg_t *req, msg_t *resp)
{
	dialog_t *d;

	d = dialog_find(dialogs, req->call_id, msg_from_tag(req), msg_to_tag(resp));
	if(d) {
		sapi_printf("dialog_create_outbound(): Warning: Dialog already exists");
		return;
	}

	d = dialog_alloc();

	d->call_id = sapi_strdup(req->call_id);
	d->ltag    = sapi_strdup(msg_from_tag(req));
	d->rtag    = sapi_strdup(msg_to_tag(resp));
	d->luri    = sapi_strdup(req->from->curl);
	d->ruri    = sapi_strdup(req->to->curl);
	d->lcseq   = sapi_to_number2(req->cseq->number);

	sapi_list_append(dialogs, (void *)d);
}


void dialog_create_inbound(list_t *dialogs, msg_t *req, msg_t *resp)
{
	dialog_t *d;

	d = dialog_find(dialogs, req->call_id, msg_from_tag(req), msg_to_tag(resp));
	if(d) {
		sapi_printf("dialog_create_inbound(): Warning: Dialog already exists");
		return;
	}

	d = dialog_alloc();

	d->call_id = sapi_strdup(req->call_id);
	d->ltag    = sapi_strdup(msg_to_tag(resp));
	d->rtag    = sapi_strdup(msg_from_tag(req));
	d->luri    = sapi_strdup(req->to->curl);
	d->ruri    = sapi_strdup(req->from->curl);
	d->rcseq   = sapi_to_number2(req->cseq->number);

	sapi_list_append(dialogs, (void *)d);
}


ret_t dialog_hangup_inbound(list_t *dialogs, msg_t *req)
{
	dialog_t *d;

	d = dialog_find(dialogs, req->call_id, msg_to_tag(req), msg_from_tag(req));
	if(!d)
		return(ERR);

	sapi_list_remove2(dialogs, (void *)d, dialog_free);

	return(OK);
}


void dialog_hangup_outbound(list_t *dialogs, msg_t *req)
{
	dialog_t *d;

	d = dialog_find(dialogs, req->call_id, msg_from_tag(req), msg_to_tag(req));
	if(!d)
		return;

	sapi_list_remove2(dialogs, (void *)d, dialog_free);

	return;
}


void dialog_status(list_t *dialogs)
{
	dialog_t *d;
	node_t *node;
	u32_t i = 0;

	sapi_printf("Dialogs: ");

	node = dialogs->start;

	while(node) {
		d = (dialog_t *)node->value;

		sapi_printf("[%03i] [%s|%s|%s] %s", i++,
			    d->call_id, d->ltag, d->rtag, d->ruri);
		node = node->next;
	}
}
