#include <sys/types.h> // open()
#include <sys/stat.h> // open()
#include <fcntl.h> // open()
#include <string.h> // strerror()
#include <errno.h> // strerror()
#include <unistd.h> // read()
#include <sys/ioctl.h> // ioctl()
#include <sys/soundcard.h> // ioctl()
#include <sapi.h>
#include <dsp.h>


#define DEVICE    "/dev/dsp"
/*
#define READ_SIZE (160)
#define FORMAT    (16)
#define CHANNELS  (2)
#define SPEED     (44100)
#define FRAG_SIZE (0x0002000c)
*/
#define READ_SIZE (160)
#define FORMAT    (16)
#define CHANNELS  (1)
#define SPEED     (8000)
#define FRAG_SIZE (0x0002000c)


typedef struct {
	void *payload;
	size_t payload_size;
} payload_t;


static dsp_read_t __read_cb = NULL;
static void *__read_param   = NULL;
static fifo_t *__writeq     = NULL;
pthread_t __dsp_reader;
pthread_t __dsp_writer;


static inline payload_t *payload_alloc()
{
	return((payload_t *)sapi_malloc(sizeof(payload_t)));
}


static inline void payload_free(payload_t *payload)
{
	if(!payload)
		return;

	sapi_free(payload->payload);
	sapi_free(payload);
}


static void *dsp_read_thread(void *arg)
{
	u8_t buf[READ_SIZE];
	s32_t n;
	int fd = -1;
	int format    = FORMAT;
	int channels  = CHANNELS;
	int speed     = SPEED;
	// int frag_size = FRAG_SIZE;


	fd = open(DEVICE, O_RDONLY);

	if(fd < 0) {
		DSP_DEBUG_PRINTF("dsp_read_thread(): %s: %s", DEVICE, strerror(errno));
		goto out;
	}

	ioctl(fd, SNDCTL_DSP_SETFMT, &format);
	ioctl(fd, SNDCTL_DSP_CHANNELS, &channels);
	ioctl(fd, SNDCTL_DSP_SPEED, &speed);
	// ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &frag_size);

	for(;;) {
		n = read(fd, buf, sizeof(buf));

		if(n != sizeof(buf)) {
			DSP_DEBUG_PRINTF("dsp_read_thread(): read returned %i", n);
		}
		else if(__read_cb){
			__read_cb((void *)buf, n, __read_param);
		}
	}

out:
	if(fd != -1)
		close(fd);

	DSP_DEBUG_PRINTF("dsp_read_thread(): exit");

	return(NULL);
}


static void *dsp_write_thread(void *arg)
{
	int fd = -1, n;
	int format    = FORMAT;
	int channels  = CHANNELS;
	int speed     = SPEED;
	// int frag_size = FRAG_SIZE;


	DSP_DEBUG_PRINTF("dsp_write_thread(): Opening device '%s'...", DEVICE);
	fd = open(DEVICE, O_WRONLY);
	DSP_DEBUG_PRINTF("dsp_write_thread(): Device '%s' open", DEVICE);

	if(fd < 0) {
		DSP_DEBUG_PRINTF("dsp_write_thread(): %s: %s", DEVICE, strerror(errno));
		goto out;
	}
	
	ioctl(fd, SNDCTL_DSP_SETFMT, &format);
	ioctl(fd, SNDCTL_DSP_CHANNELS, &channels);
	ioctl(fd, SNDCTL_DSP_SPEED, &speed);
	// ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &frag_size);

	while(__writeq) {

		payload_t *p = (payload_t *)sapi_fifo_get(__writeq);

		n = write(fd, p->payload, p->payload_size);

		if(n != p->payload_size) {
			DSP_DEBUG_PRINTF("dsp_write_thread(): write returned %i, payload_size is %i", n, p->payload_size);
		}
		else {
			// DSP_DEBUG_PRINTF("dsp_write_thread(): wrote payload of %i bytes", n);
		}

		payload_free(p);

		if(!sapi_fifo_nnodes(__writeq)) {
			DSP_DEBUG_PRINTF("dsp_write_thread(): write queue is empty");
		}
	}

out:
	if(fd != -1)
		close(fd);

	DSP_DEBUG_PRINTF("dsp_write_thread(): exit");
	
	return(NULL);
}


void dsp_init(dsp_read_t read, void *arg)
{
	__read_cb    = read;
	__read_param = arg;
	__writeq     = sapi_fifo_alloc();
	
	pthread_create(&__dsp_reader, NULL, dsp_read_thread, NULL);
	pthread_create(&__dsp_writer, NULL, dsp_write_thread, NULL);

	sapi_logf("DSP service is up and running");
}


void dsp_write(void *payload, size_t payload_size)
{
	payload_t *p;

	if(!__writeq || !payload || !payload_size) {
		DSP_DEBUG_PRINTF("dsp_write(): empty payload or DSP not initialized");
		return;
	}

	p = payload_alloc();
	
	p->payload = sapi_malloc(payload_size);
	memcpy(p->payload, payload, payload_size);

	p->payload_size = payload_size;

	sapi_fifo_put(__writeq, p);
}


void dsp_status(void)
{
	if(!__writeq)
		return;

	sapi_printf("DSP Write Queue:");
	sapi_fifo_status(__writeq);
}
