#include <stdlib.h>
#include <unistd.h>
#include <version.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include "im.h"


static void err(const msg_t *req, const u8_t *reason, void *arg, u8_t *sarg)
{
	sapi_logf("Beep failed: %s", reason);
}


static void resp(const msg_t *req, const msg_t *resp, void *arg, u8_t *sarg)
{
	sapi_logf("%s Response: %s (%s)",
		  req->request->method,
		  resp->response->statuscode,
		  resp->response->reasonphrase);
}


void im_beep(im_t *im, const address_t *addr)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(im->local->url->host, im->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), im->global->url->host);
	tp_t *tp        = tp_alloc();
	
	tp->host  = sapi_strdup(im->proxy->host);
	tp->port  = im->proxy->port;
	tp->proto = UDP;

	msg_via_addparam(via, "branch", branch);

	if(addr) {
		
		msg_set_request(msg, "BEEP", addr->url);
		msg_add_via(msg, via);
		msg_set_to(msg, addr);
		msg_set_from(msg, im->global);
		msg_set_call_id(msg, call_id);
		msg_set_cseq(msg, im->cseq++, "BEEP");

		ctrans_req(tp, msg, resp, err, NULL, NULL);
	}

	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}
