#include <sapi.h>
#include <msg.h>
#include <gtk/gtk.h>
#include "im.h"


static GtkWidget *_msgview = NULL;
static GtkWidget *_msgedit = NULL;
static GtkWidget *_log     = NULL;
static im_t *_im = NULL;


void address(GtkWidget *widget, GtkWidget *entry)
{
	if(_im)
		im_set_remote(_im, gtk_entry_get_text(GTK_ENTRY(entry)));
}


static void send(GtkWidget *widget, gpointer data)
{
	u8_t *message;

	if(!_im)
		return;
	
	if(!_im->remote) {
		sapi_logf("Remote address is not set");
		return;
	}

	if(!sapi_strlen(gtk_editable_get_chars(GTK_EDITABLE(_msgedit), 0, -1))) {
		sapi_logf("Please enter something!");
		return;
	}

	message = sapi_sprintf("%s\n" , gtk_editable_get_chars(GTK_EDITABLE(_msgedit), 0, -1));
	gui_msg(FALSE, _im->str_global, message);
	im_message(_im, _im->remote, message);
	sapi_free(message);

	gtk_text_set_point(GTK_TEXT(_msgedit), 0);
	gtk_text_forward_delete(GTK_TEXT(_msgedit), gtk_text_get_length(GTK_TEXT(_msgedit)));
}


static void beep(GtkWidget *widget, gpointer data)
{
	if(!_im)
		return;
	
	if(!_im->remote) {
		sapi_logf("Remote address is not set");
		return;
	}

	im_beep(_im, _im->remote);
}


static void close(GtkWidget *widget, gpointer data)
{
	_log = NULL;
	_msgview = NULL;
	g_print("Goodbye\n");

	gtk_main_quit();
}


static GtkWidget *mk_window(void)
{
	GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	gtk_widget_set_usize(window, 800, 800);
	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
	gtk_signal_connect(GTK_OBJECT (window), "destroy", GTK_SIGNAL_FUNC(close), NULL);
	gtk_window_set_title(GTK_WINDOW(window), "IM");
	gtk_container_set_border_width(GTK_CONTAINER (window), 0);

	return(window);
}


static GtkWidget *mk_box1(GtkWidget *window)
{
	GtkWidget *box1 = gtk_vbox_new(FALSE, 0);

	gtk_container_add(GTK_CONTAINER(window), box1);
	gtk_widget_show (box1);
	
	return(box1);
}


static GtkWidget *mk_box2(GtkWidget *box1)
{
	GtkWidget *box2 = gtk_vbox_new(FALSE, 10);

	gtk_container_set_border_width(GTK_CONTAINER(box2), 10);
	gtk_box_pack_start(GTK_BOX(box1), box2, TRUE, TRUE, 0);
	gtk_widget_show(box2);

	return(box2);
}


static GtkWidget *mk_table(GtkWidget *box2)
{
	GtkWidget *table = gtk_table_new(2, 2, FALSE);

	gtk_table_set_row_spacing(GTK_TABLE(table), 0, 2);
	gtk_table_set_col_spacing(GTK_TABLE(table), 0, 2);
	gtk_box_pack_start(GTK_BOX(box2), table, TRUE, TRUE, 0);
	gtk_widget_show(table);

	return(table);
}


static GtkWidget *mk_textbox(GtkWidget *table, gint editable)
{
	GtkWidget *text = gtk_text_new(NULL, NULL);
	GtkWidget *vscrollbar;

	gtk_text_set_editable(GTK_TEXT(text), editable);
	gtk_table_attach(GTK_TABLE(table), text, 0, 1, 0, 1,
			 GTK_EXPAND | GTK_SHRINK | GTK_FILL,
			 GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
	gtk_widget_show (text);
	
	/* Add a vertical scrollbar to the GtkText widget */
	vscrollbar = gtk_vscrollbar_new (GTK_TEXT(text)->vadj);
	gtk_table_attach(GTK_TABLE(table), vscrollbar, 1, 2, 0, 1,
			 GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
	gtk_widget_show (vscrollbar);

	return(text);
}


static void mk_separator(GtkWidget *box1)
{
	GtkWidget *separator = gtk_hseparator_new();

	gtk_box_pack_start(GTK_BOX(box1), separator, FALSE, TRUE, 0);
	gtk_widget_show(separator);
}


static GtkWidget *mk_box3(GtkWidget *box1)
{
	GtkWidget *box3 = gtk_vbox_new(FALSE, 10);
	gtk_container_set_border_width(GTK_CONTAINER (box3), 10);
	gtk_box_pack_start(GTK_BOX (box1), box3, FALSE, TRUE, 0);
	gtk_widget_show(box3);

	return(box3);
}


static void mk_entry(GtkWidget *box)
{
	GtkWidget *entry = gtk_entry_new_with_max_length(50);
	
	gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(address), entry);

	gtk_box_pack_start(GTK_BOX(box), entry, TRUE, TRUE, 0);
	gtk_widget_show(entry);
}


static void mk_button(GtkWidget *box, const char *name, void *func)
{
	GtkWidget *button = gtk_button_new_with_label (name);

	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(func), NULL);

	gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
	gtk_widget_show(button);
}


void gui_init(im_t *im)
{
	GtkWidget *window, *box1, *box2, *box3, *table;

	_im = im;
	
	window   = mk_window();
	box1     = mk_box1(window);

	box2     = mk_box2(box1);
	table    = mk_table(box2);
	_msgview = mk_textbox(table, FALSE);

	box2     = mk_box2(box1);
	table    = mk_table(box2);
	_msgedit = mk_textbox(table, TRUE);

	box3     = mk_box3(box1);
	mk_entry(box3);
	mk_button(box3, "Send", send);
	mk_button(box3, "Beep", beep);

	box2     = mk_box2(box1);
	table    = mk_table(box2);
	_log     = mk_textbox(table, FALSE);

	mk_separator(box1);
	box3   = mk_box3(box1);
	mk_button(box3, "Quit", close);
	
	gtk_widget_show (window);
}


void gui_msg(bool_t remote, const char *who, const char *text)
{
	GdkColormap *cmap;
	GdkColor color;
	u8_t *head;

	if(!_msgview)
		return;

	cmap = gdk_colormap_get_system();

	if(remote) {
		color.red = 0xffff;
		color.green = 0;
		color.blue = 0;
	}
	else {
		color.red = 0;
		color.green = 0x8888;
		color.blue = 0;
	}

	if(!gdk_color_alloc(cmap, &color)) {
		g_error("couldn't allocate color");
	}

	head = sapi_sprintf("%s>\n", who);
	gtk_text_insert(GTK_TEXT(_msgview), NULL, &color, NULL, head, -1);
	sapi_free(head);
	gtk_text_insert(GTK_TEXT(_msgview), NULL, NULL, NULL, text, -1);
}


void gui_beep(const char *who)
{
	GtkWidget *dialog, *label;

	dialog = gtk_dialog_new();

	label = gtk_label_new(who);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	gtk_widget_show(dialog);
}


void gui_log(const char *msg)
{
	GdkColormap *cmap;
	GdkColor color;

	if(!_log)
		return;

	cmap = gdk_colormap_get_system();
	color.red = 0;
	color.green = 0;
	color.blue = 0xffff;
	if(!gdk_color_alloc(cmap, &color)) {
		g_error("couldn't allocate color");
	}

	gtk_text_insert(GTK_TEXT(_log), NULL, &color, NULL, msg, -1);
}
