#include <stdio.h> // fprintf()
#include <stdlib.h> // for getenv() and rand()
#include <sys/types.h> // for open()
#include <sys/stat.h> // for open()
#include <fcntl.h> // for open()
#include <string.h> // for strerror()
#include <errno.h> // for strerror()
#include <unistd.h>
#include <time.h>
#include <version.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include "im.h"


#define DOMAINNAME "sxdesign.com"
#define PROXY      "sip:sxdesign.com"


im_t *im_init(void)
{
	im_t *im = (im_t *)sapi_malloc((u32_t)sizeof(im_t));
	u8_t hostname[255];

	gethostname(hostname, sizeof(hostname));
	
	im->str_local  = sapi_sprintf("sip:%s@%s.%s:%u", getlogin(), hostname, DOMAINNAME, (u16_t)getpid());
	im->str_global = sapi_sprintf("sip:%s@%s", getlogin(), DOMAINNAME);
	im->str_proxy  = sapi_strdup(PROXY);

	im->local  = msg_address_alloc();
	im->global = msg_address_alloc();	
	im->proxy  = msg_url_alloc();

	if(msg_address_parse(im->local, im->str_local) != OK) {
		fprintf(stderr, "Error parsing local address: '%s'", im->str_local);
		goto out_err;
	}

	if(msg_address_parse(im->global, im->str_global) != OK) {
		fprintf(stderr, "Error parsing global address: '%s'", im->str_global);
		goto out_err;
	}

	if(msg_url_parse(im->proxy, im->str_proxy) != OK) {
		fprintf(stderr, "Error parsing proxy URI: '%s'", im->str_proxy);
		goto out_err;
	}	
#if 0
	fprintf(stderr, "SIP IM Version: " SIPD_VERSION);
	fprintf(stderr, "Local Address : '%s'", im->str_local);
	fprintf(stderr, "Global Address: '%s'", im->str_global);
	fprintf(stderr, "Proxy URI     : '%s'", im->str_proxy);
#endif
	return(im);

out_err:
	im_free(im);
	return(NULL);
}


void im_free(im_t *im)
{
	if(!im)
		return;

	sapi_free(im->str_local);
	sapi_free(im->str_global);
	sapi_free(im->str_proxy);

	msg_address_free(im->local);
	msg_address_free(im->global);
	msg_url_free(im->proxy);

	msg_address_free(im->remote);

	sapi_free(im);
}


void im_set_remote(im_t *im, const u8_t *remote_str)
{
	if(!im || !remote_str) {
		sapi_logf("im_set_remote_str(): im or remote_str is NULL");
		return;
	}

	if(im->remote)
		msg_address_free(im->remote);

	im->remote = msg_address_alloc();
	if(msg_address_parse(im->remote, remote_str) != OK) {
		msg_address_free(im->remote);
		im->remote = NULL;
		sapi_logf("Error parsing address: '%s'", remote_str);
		return;
	}

	sapi_logf("Remote address is '%s'", remote_str);
}
