#include <stdio.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include <strans.h>
#include <gtk/gtk.h>
#include "im.h"


static void log(u8_t *msg)
{
	gdk_threads_enter();
	gui_log(msg);
	gdk_threads_leave();
}


static void err(const u8_t *reason, void *arg)
{
        sapi_logf("Server Transaction Error : %s", reason);
}


static void req(const msg_t *req, const msg_t *oreq, const u8_t *hkey, const u8_t *ohkey, void *arg)
{
	msg_t *resp;

	if(sapi_strcmp(req->request->method, "MESSAGE")) {
                
		u8_t *from;

                resp = msg_response(req, "200", "OK");
                strans_resp(hkey, resp);
                msg_deref(resp);                

		from = msg_url_cannonical(req->from->url);
		
		if(from && req->body) {
			gdk_threads_enter();
			gui_msg(TRUE, from, req->body);
			gdk_threads_leave();
		}

		sapi_free(from);
        }
	if(sapi_strcmp(req->request->method, "BEEP")) {
                
		u8_t *from;

                resp = msg_response(req, "200", "OK");
                strans_resp(hkey, resp);
                msg_deref(resp);                

		from = msg_url_cannonical(req->from->url);
		
		if(from) {
			gdk_threads_enter();
			gui_beep(from);
			gdk_threads_leave();
		}

		sapi_free(from);
        }
	else {
                sapi_logf("Got %s request", req->request->method);
                
                msg_deref(resp);
        }
}


static void tp_recv(const tp_t *tp, const msg_t *msg, void *param)
{
        if(msg->response) {             
                ctrans_resp(msg);
        }
        else if(msg->request) {
                strans_req(tp, msg, req, err, NULL);
        }
}


static void autoregister(void *arg, u8_t *sarg)
{
        im_t *im = (im_t *)arg;

        im_register(im, 360);
        sapi_oneshot(3000, autoregister, im, NULL);
}


int main(int argc, char *argv[])
{
	im_t *im = im_init();
	if(!im)
                return(-1);

	g_thread_init(NULL);
	gtk_init(&argc, &argv);	
	gui_init(im);

	sapi_log_init2(log);
	sapi_oneshot_init();

	tp_init(im->local->url->port, tp_recv, im);
        ctrans_init();
        strans_init();

	autoregister(im, NULL);

	gdk_threads_enter();
	gtk_main();
	gdk_threads_leave();

	im_register(im, 0);
	
	return(0);       
}
