#include <sapi.h>
#include <msg.h>


static u8_t *quote_find(const u8_t *qstring)
{
	u8_t *quote;
	u32_t i = 1;


	quote = sapi_strchr(qstring, '"');
	if(quote == qstring)
		return(quote);

	if(quote == NULL)
		return(NULL);

	while(1) {

		if(*(quote - i) == '\\')
			i++;
		else {
			if (i%2 == 1)
				return(quote);

			quote = sapi_strchr(quote+1, '"');
			if (quote==NULL)
				return(NULL);
			i = 1;
		}
		if (quote-i==qstring+1) {

			if (i%2==0)
				return(quote);
			else {
				quote = sapi_strchr(quote+1,'"');
				if (quote == NULL)
					return(NULL);
				i = 1;
			}

		}
	}
	return(NULL);
}


address_t *msg_address_alloc()
{
	return((address_t *)sapi_malloc((u32_t)sizeof(address_t)));
}


void msg_address_free(void *ptr)
{
	address_t *address = (address_t *)ptr;

	if(!address)
		return;

	sapi_free(address->displayname);
	msg_url_free(address->url);
	sapi_list_free(address->params, msg_param_free);
	sapi_free(address);
}


address_t *msg_address_dup(const address_t *address)
{
	address_t *new_address;

	if(!address)
		return(NULL);

	new_address = msg_address_alloc();
	
	if(address->displayname)
		new_address->displayname = sapi_strdup(address->displayname);

	if(address->url)
		new_address->url = msg_url_dup(address->url);

	if(address->params)
		new_address->params = msg_param_list_dup(address->params);

	return(new_address);
}


void msg_address_addparam(address_t *address, const u8_t *name, const u8_t *value)
{
	if(!address || !name || !value) {
		MSG_DEBUG_PRINTF("msg_address_addparam(): address, name of value is NULL");
		return;
	}

	if(!address->params)
		address->params = sapi_list_alloc();

	msg_param_list_add_param(address->params, name, value);
}


void msg_address_addparam_u32(address_t *address, const u8_t *name, u32_t value)
{
	if(!address || !name) {
		MSG_DEBUG_PRINTF("msg_address_addparam_u32(): address or name is NULL");
		return;
	}

	if(!address->params)
		address->params = sapi_list_alloc();

	msg_param_list_add_param_u32(address->params, name, value);
}


u8_t *msg_address_getparam(const address_t *address, const u8_t *name)
{
	if(!address || !name) {
		MSG_DEBUG_PRINTF("msg_address_getparam(): address or name is NULL");
		return(NULL);
	}

	return(msg_param_list_get_param(address->params, name));
}


ret_t msg_address_parse(address_t *address, const u8_t *address_str)
{
	u8_t *displayname;
	u8_t *url;
	u8_t *url_end;
	u8_t *gen_params;
	u8_t *host;
	u8_t *first;
	u8_t *second;
	u8_t *url_str;


	displayname = sapi_strchr(address_str, '"');
	url = sapi_strchr(address_str, '<');

	if(displayname > url)
		displayname = NULL;

	if((displayname == NULL) && (url != NULL)) {
		if(address_str != url) {
			if(url - address_str + 1 < 2) {
				MSG_DEBUG_PRINTF("msg_address_parse(): Invalid URL (to short)");
				return(ERR);
			}

			address->displayname = sapi_strndup(address_str, url - address_str - 1);
		}
		url++;
	}
	else {
		if((displayname != NULL) && (url != NULL)) {
			first  = quote_find(address_str);
			second = quote_find(first+1);
			if (second==NULL) {
				MSG_DEBUG_PRINTF("msg_address_parse(): Second qoute is missing");
				return(ERR);
			}
			if ((first>url)) {
				MSG_DEBUG_PRINTF("msg_address_parse(): First quote beyond url start");
				return(ERR);
			}

			if (second-first+2>=2) {
				address->displayname = sapi_strndup(first, second - first + 1);
			}
			url = sapi_strchr(second+1,'<');
			if (url==NULL) {
				MSG_DEBUG_PRINTF("msg_address_parse(): No URL");
				return(ERR);
			}
			url++;
		}
		else
			url = (u8_t *)address_str;
	}

	url_end = sapi_strchr(url,'>');

	if (url_end == NULL) {
		host = sapi_strchr(url,'@');
		if (host != NULL)
			gen_params = sapi_strchr(host,';');
		else
			gen_params = sapi_strchr(url,';');
		if (gen_params != NULL)
			url_end = gen_params - 1;
		else
			url_end = url + sapi_strlen(url);
	}
	else {
		gen_params = sapi_strchr(url_end, ';');
		url_end--;
	}

	if(gen_params) {
		address->params = sapi_list_alloc();
		if (msg_param_list_parse(address->params, gen_params + 1, gen_params + sapi_strlen(gen_params), ';') == ERR) {
			MSG_DEBUG_PRINTF("msg_address_parse(): parsing param list failed");
			return(ERR);
		}
	}

	if (url_end-url + 2 < 7) {
		MSG_DEBUG_PRINTF("msg_address_parse(): URL to short");
		return(ERR);
	}

	address->url  = msg_url_alloc();
	url_str = sapi_strndup(url , url_end - url + 1);
	
	if(msg_url_parse(address->url, url_str) != OK) {
		MSG_DEBUG_PRINTF("msg_address_parse(): parsing URL failed");
		sapi_free(url_str);
		return(ERR);
	}

	sapi_free(url_str);

	return(OK);
}


u8_t *msg_address_str(const address_t *address)
{
	u8_t *str = NULL, *tmp;

	if(!address || !address->url) {
		MSG_DEBUG_PRINTF("msg_address_str(): address or address->url is NULL");
		return(NULL);
	}

	if(address->displayname)
		sapi_saprintf(&str, "%s ", address->displayname);

	tmp = msg_url_str(address->url);
	sapi_saprintf(&str, "<%s>", tmp);
	sapi_free(tmp);

	if(address->params) {
		tmp = msg_param_list_str(address->params, ';');
		sapi_saprintf(&str, ";%s", tmp);
		sapi_free(tmp);
	}
	
	return(str);
}


u8_t *msg_address_display_str(const address_t *address)
{
	u8_t *str;

	if(!address || !address->url) {
		MSG_DEBUG_PRINTF("msg_address_str(): address or address->url is NULL");
		return(NULL);
	}

	if(address->displayname)
		str = sapi_strdup(address->displayname);
	else
		str = msg_url_cannonical(address->url);

	return(str);
}
