#include <sapi.h>
#include <msg.h>


static ret_t dup_apply(void *value, u32_t n, void *arg1, void *arg2)
{
	param_t *param = (param_t *)value;
	list_t *params = (list_t *)arg1;

	if(!param || !params)
		return(ERR);

	sapi_list_append(params, msg_param_dup(param));

	return(OK);
}


list_t *msg_param_list_dup(const list_t *params)
{
	list_t *new_params;

	if(!params)
		return(NULL);

	new_params = sapi_list_alloc();

	sapi_list_apply((list_t *)params, dup_apply, new_params, NULL);

	return(new_params);
}


void msg_param_list_add_param(list_t *params, const u8_t *name, const u8_t *value)
{
	param_t *param;

	if(!params || !name || !value)
		return;

	param = msg_param_alloc();

	param->name  = sapi_strdup(name);
	param->value = sapi_strdup(value);

	sapi_list_append(params, (void *)param);
}


void msg_param_list_add_param_u32(list_t *params, const u8_t *name, u32_t value)
{
	u8_t *str;

	if(!params || !name)
		return;

	str = sapi_sprintf("%i", value);
	msg_param_list_add_param(params, name, str);
	sapi_free(str);
}


static bool_t param_find(void *value, void *arg)
{
	param_t *param = (param_t *)value;
	u8_t *name = (u8_t *)arg;

	if(!param || !name)
		return(FALSE);

	return(sapi_strcmp_ci(param->name, name));
}


u8_t *msg_param_list_get_param(const list_t *params, const u8_t *name)
{
	param_t *param;

	if(!params || !name)
		return(NULL);

	param = (param_t *)sapi_list_find((list_t *)params, param_find, (void *)name);
	if(!param)
		return(NULL);

	return(param->value);
}


ret_t msg_param_list_parse(list_t *params, const u8_t *start, const u8_t *end, u8_t sep_sign)
{
	param_t *param;
	u8_t *equal, *sep, *param_end;

	if(!params || !start || !end || end <= start) {
		MSG_DEBUG_PRINTF("msg_param_list_parse(): sanity check failed (%i|%i|%i)", params, start, end);
		return(ERR);
	}

	while(*start) {

		equal     = sapi_strchr(start, '=');
		sep = sapi_strchr(start, sep_sign);

		if(!equal) {
			MSG_DEBUG_PRINTF("msg_param_list_parse(): equal-sign missing");
			return(ERR);
		}
		
		if(sep && (sep < equal)) {
			MSG_DEBUG_PRINTF("msg_param_list_parse(): malformed parameter list (sep < equal)");
			return(ERR);
		}

		if(!sep)
			param_end = (u8_t *)end;
		else
			param_end = sep;

		param = msg_param_alloc();

		param->name  = sapi_lower(sapi_strndup(start, equal - start));
		param->value = sapi_strndup(equal + 1, param_end - equal - 1);
		sapi_list_append(params, param);

		if(param_end == end)
			break;

		start = param_end + 1;
	}

	return(OK);
}


ret_t param_str(void *value, u32_t n, void *arg1, void *arg2)
{
	param_t *param = (param_t *)value;
	u8_t **str = (u8_t **)arg1;
	u8_t *sep  = (u8_t *)arg2;

	if(!param || !str || !sep) {
		MSG_DEBUG_PRINTF("param_str(): param, str or sep is NULL");
		return(ERR);
	}

	if(n)
		sapi_saprintf(str, "%c", *sep);

	sapi_saprintf(str, "%s=%s", param->name, param->value);

	return(OK);
}


u8_t *msg_param_list_str(const list_t *params, u8_t sep_sign)
{
	u8_t *str = NULL;

	if(!params) {
		MSG_DEBUG_PRINTF("msg_param_list_str(): params is NULL");
		return(NULL);
	}

	/* Handles empty lists */
	sapi_saprintf(&str, "");
	sapi_list_apply((list_t *)params, param_str, &str, &sep_sign);

	return(str);	
}
