#include <sapi.h>
#include <msg.h>


request_t *msg_request_alloc(void)
{
	return((request_t *)sapi_malloc((u32_t)sizeof(request_t)));
}


void msg_request_free(request_t *request)
{
	if(!request)
		return;

	sapi_free(request->method);
	msg_url_free(request->url);
	sapi_free(request->ver);
	sapi_free(request);

}


ret_t msg_request_parse(request_t *request, const u8_t *request_str)
{
	u8_t *sp1, *sp2, *url_str;
	
	sp1 = sapi_strchr(request_str, ' ');
	if(!sp1) {
		MSG_DEBUG_PRINTF("msg_request_parse(): missing space1");
		return(ERR);
	}

	request->method = sapi_strndup(request_str, (u32_t)(sp1 - request_str));

	sp2 = sapi_strchr(sp1 + 1, ' ');
	if(!sp2) {
		MSG_DEBUG_PRINTF("msg_request_parse(): missing space2");
		return(ERR);
	}

	request->url = msg_url_alloc();
	url_str = sapi_strndup(sp1 + 1, (u32_t)(sp2 - sp1 - 1));
	
	if(msg_url_parse(request->url, url_str) != OK) {
		MSG_DEBUG_PRINTF("msg_request_parse(): Error parsing Request URL");
		sapi_free(url_str);
		return(ERR);
	}
	sapi_free(url_str);

	request->ver = sapi_strdup(sp2 + 1);

	if(sapi_strcmp(request->ver, SIP_VERSION) == FALSE) {
		MSG_DEBUG_PRINTF("msg_request_parse(): Unsupported SIP Version: '%s'", request->ver);
		return(ERR);
	}

	return(OK);
}


u8_t *msg_request_str(const request_t *request)
{
	u8_t *str = NULL, *tmp;

	if(!request || !request->method || !request->url || !request->ver) {
		MSG_DEBUG_PRINTF("request_str(): request, request->method, request->url or request->ver is NULL");
		goto out_err;
	}

	sapi_saprintf(&str, "%s ", request->method);

	if((tmp = msg_url_str(request->url))) {
		sapi_saprintf(&str, "%s ", tmp);
		sapi_free(tmp);
	}
	else {
		MSG_DEBUG_PRINTF("request_str(): msg_url_str(): failed");
		goto out_err;
	}

	sapi_saprintf(&str, "%s", request->ver);

	return(str);

out_err:
	sapi_free(str);
	return(NULL);
}
