#include <sapi.h>
#include <msg.h>


response_t *msg_response_alloc(void)
{
	return((response_t *)sapi_malloc((u32_t)sizeof(response_t)));
}


void msg_response_free(response_t *response)
{
	if(!response)
		return;

	sapi_free(response->ver);
	sapi_free(response->statuscode);
	sapi_free(response->reasonphrase);
	sapi_free(response);
}


ret_t msg_response_parse(response_t *response, const u8_t *response_str)
{
	u8_t *sp1, *sp2;

	sp1 = sapi_strchr(response_str, ' ');
	if(!sp1) {
		MSG_DEBUG_PRINTF("msg_response_parse(): First Space is missing");
		return(ERR);
	}

	response->ver = sapi_strndup(response_str, (u32_t)(sp1 - response_str));

	if(sapi_strcmp(response->ver, SIP_VERSION) == FALSE) {
		MSG_DEBUG_PRINTF("msg_response_parse(): Unsupported SIP Version: '%s'", response->ver);
		return(ERR);
	}

	sp2 = sapi_strchr(sp1 + 1, ' ');
	if(!sp2) {
		MSG_DEBUG_PRINTF("msg_response_parse(): Second Space is missing");
		return(ERR);
	}

	response->statuscode = sapi_strndup(sp1 + 1, (u32_t)(sp2 - sp1 - 1));
	if(sapi_is_numeric(response->statuscode) == FALSE) {
		MSG_DEBUG_PRINTF("msg_response_parse(): Statuscode is not numeric");
		return(ERR);
	}

	response->reasonphrase = sapi_strdup(sp2 + 1);

	return(OK);
}


u8_t *msg_response_str(const response_t *response)
{
	u8_t *str = NULL;

	if(!response || !response->ver || !response->statuscode || !response->reasonphrase) {
		MSG_DEBUG_PRINTF("request_str(): response, version, statuscode or resonphrase is NULL");
		return(NULL);
	}

	sapi_saprintf(&str, "%s %s %s", response->ver, response->statuscode, response->reasonphrase);

	return(str);
}
