#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include "sapi.h"


#define LOGFILE "/var/log/sipd.log"


static fifo_t *__logq = NULL;
static pthread_t _thread;


typedef struct {
	time_t time;
	u8_t *msg;
} logentry_t;


static logentry_t *logentry_alloc(void)
{
	return((logentry_t *)sapi_malloc(sizeof(logentry_t)));
}


static void logentry_free(logentry_t *le)
{
	if(!le)
		return;

	sapi_free(le->msg);
	sapi_free(le);
}


static void *log_thread(void *arg)
{
	u8_t *logfile = (u8_t *)arg;
	int fd;

	if(!logfile)
		return(NULL);

	// fd = open(logfile, O_WRONLY | O_APPEND | O_SYNC);
	fd = open(logfile, O_WRONLY | O_APPEND);

        while(__logq) {
                logentry_t *le = (logentry_t *)sapi_fifo_get(__logq);

		if(fd != -1) {
			u8_t timestr[20];
			u8_t *str;
			
			strftime(timestr, sizeof(timestr), "%a %b %d %H:%M:%S", localtime(&le->time));
			timestr[sizeof(timestr) - 1] = '\0';

			str = sapi_sprintf("%s %s\n", timestr, le->msg);
			
			write(fd, (void *)str, (size_t)sapi_strlen(str));
			sapi_free(str);
		}
		
		logentry_free(le);
        }

	if(fd != -1)
		close(fd);

	return(NULL);
}


static void *log_thread2(void *arg)
{
	sapicb_log cb = (sapicb_log)arg;

        while(__logq) {

                logentry_t *le = (logentry_t *)sapi_fifo_get(__logq);
		u8_t timestr[20];
		u8_t *str;
			
		strftime(timestr, sizeof(timestr), "%a %b %d %H:%M:%S", localtime(&le->time));
		timestr[sizeof(timestr) - 1] = '\0';
		
		str = sapi_sprintf("%s %s\n", timestr, le->msg);
		
		if(cb)
			cb(str);
		
		sapi_free(str);
		
		logentry_free(le);
        }
	
	return(NULL);
}


void sapi_log_init(const u8_t *logfile)
{
	__logq = sapi_fifo_alloc();
	pthread_create(&_thread, NULL, log_thread, (void *)logfile);

	sapi_logf("Log service is up and running");
}


void sapi_log_init2(sapicb_log cb)
{
	__logq = sapi_fifo_alloc();
	pthread_create(&_thread, NULL, log_thread2, cb);

	sapi_logf("Log service is up and running");	
}


void sapi_logf(const u8_t *fmt, ...)
{
        /* Guess we need no more than 100 bytes. */
        s32_t n;
        u32_t size = 100;
        va_list ap;
	logentry_t *le;

        if(!fmt || !__logq)
                return;

	le = logentry_alloc();

	le->time = time(NULL);
        le->msg = (u8_t *)sapi_malloc(size);

        for(;;) {

                /* Try to print in the allocated space. */
                va_start(ap, (char *)fmt);
                n = (s32_t)vsnprintf((char *)le->msg, (size_t)size, (const char *)fmt, ap);
                va_end(ap);

                /* If that worked, return the string. */
                if(n > -1 && n < size)
                        break;

                /* Else try again with more space. */
                if(n > -1)           /* glibc 2.1 */
                        size = n + 1; /* precisely what is needed */
                else                 /* glibc 2.0 */
                        size *= 2;    /* twice the old size */
                le->msg = (u8_t *)sapi_realloc((void *)le->msg, size);
        }

	sapi_fifo_put(__logq, (void *)le);
}


void sapi_log_status(void)
{
	if(!__logq)
		return;

	sapi_fifo_status(__logq);
}


u8_t *sapi_log_status_str(void)
{
	if(!__logq)
		return(sapi_sprintf("sapi_log_status_str(): __logq is NULL"));

	return(sapi_fifo_status_str(__logq));
}
