#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include "sapi.h"


static u32_t __nmalloced = 0;
static pthread_mutex_t __mutex = PTHREAD_MUTEX_INITIALIZER;


void *sapi_malloc(u32_t size)
{
	void *newptr;

	newptr = malloc((size_t)size);

	if(!newptr) {
		SAPI_DEBUG_PRINTF("sapi_malloc(): size is %u", size);
		sapi_bluesmoke("sapi_malloc(): malloc returned NULL");
	}

	memset(newptr, (int)0, (size_t)size);

	pthread_mutex_lock(&__mutex);
	__nmalloced++;
	pthread_mutex_unlock(&__mutex);

	return(newptr);
}


void *sapi_realloc(void *ptr, u32_t size)
{
	void *newptr;
	
	if(!ptr) {
		pthread_mutex_lock(&__mutex);
		__nmalloced++;
		pthread_mutex_unlock(&__mutex);
	}

	newptr = realloc(ptr, (size_t)size);

	if(!newptr)
		sapi_bluesmoke("sapi_realloc(): realloc returned NULL");

	return(newptr);
}


void sapi_free(void *ptr)
{
	if(ptr) {
		free(ptr);
		pthread_mutex_lock(&__mutex);
		__nmalloced--;
		pthread_mutex_unlock(&__mutex);
	}
}


u32_t sapi_nmalloced(void)
{
	u32_t n;

	pthread_mutex_lock(&__mutex);
	n = __nmalloced;
	pthread_mutex_unlock(&__mutex);

	return(n);
}
