/**
 * @file sdp_dbg.c Implements SDP debug code.
 *
 * Copyright 2002-2003 SX Design AS <www.sxdesign.com>.  All rights reserved.
 */
#include <sapi.h>
#include <sdp.h>
#include "media.h"


/**
 * Dump an attrib object (list_apply callback).
 *
 * @param value Node value.
 * @param arg   Callback argument.
 */
static ret_t attrib_dump(void *value, u32_t n, void *arg1, void *arg2)
{
	sdp_attrib_t *attrib = (sdp_attrib_t *)value;

	(void)n;
	(void)arg1;
	(void)arg2;

	SDP_DEBUGFx("");
	SDP_DEBUGFx("  rtp_pt: %u", attrib->rtp_pt);
	SDP_DEBUGFx("  rtpmap: %s", attrib->rtpmap);
	SDP_DEBUGFx("  clock:  %d", attrib->clockrate);
	SDP_DEBUGFx("  fmtp:   %s", attrib->fmtp);

	return(OK);
}


/**
 * Dump an SDP struct to the debug port.
 *
 * @param sdp SDP struct to be dumped.
 */
void sdp_dump(const sdp_t *sdp)
{
	SDP_DEBUGFx("SDP object at 0x%04x", sdp);

	SDP_DEBUGFx(" host:   '%s:%u'", sdp->host, sdp->media->port);
	SDP_DEBUGFx(" type:   '%s'", sdp->media->type);
	SDP_DEBUGFx(" transp: '%s'", sdp->media->transp);

	SDP_DEBUGFx(" %u media attribs:", sapi_list_nnodes(sdp->media->attribs));

	(void)sapi_list_apply(sdp->media->attribs, attrib_dump, NULL, NULL);
}
