/**
 * @file sdp_test.c Test code for SDP - Session Description Protocol (RFC 2327)
 *
 * Copyright 2002-2003 SX Design AS <www.sxdesign.com>.  All rights reserved.
 */
#include <sapi.h>
#include <sdp.h>
#include "media.h"


static const u8_t sdp_body[] =  
"v=0\r\n"
"o=- 0 0 IN IP4 127.0.0.1\r\n"
"s=-\r\n"
"c=IN IP4 127.0.0.1\r\n"
"t=0 0\r\n"
"m=audio 5004 RTP/AVP 4 0 8\r\n"
"a=rtpmap:4 PCMA/8000\r\n"
"a=rtpmap:0 PCMU/8000\r\n"
"a=rtpmap:8 GSM/8000\r\n"
;


int sdp_test(const u8_t *body)
{
	u8_t *str;
	sdp_t *s = sdp_parse(sdp_body);
	sapi_printf("SDP body first pass:\n%s", body);

	if(!s) {
		sapi_printf("failed to parse SDP body");
		return(1);
	}

	sdp_dump(s);

	str = sdp_str(s);
	if(!str) {
		sapi_printf("failed to create SDP string");
		return(1);
	}

	sapi_printf("SDP body second pass:\n%s", str);

	sapi_free(str);
	sdp_free(s);

	/* success */
	return(0);
}


int main(int argc, char **argv)
{
	int ret;

	ret = sdp_test(sdp_body);

	return(ret);
}
