#include <unistd.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include <strans.h>
#include <ldb.h>
#include <status.h>
#include "sipd.h"


static void err(const u8_t *reason, void *arg)
{
	SIPD_DEBUG_PRINTF("Server Transaction Error : %s", reason);
}


static void req(const msg_t *req, const msg_t *oreq, const u8_t *hkey, const u8_t *ohkey, void *arg)
{
	sipd_t *sipd = (sipd_t *)arg;

        if(sapi_strcmp(req->request->method, "REGISTER")) {
		
                sipd_registrar(req, hkey);
	}
        else if(sapi_strcmp(req->request->method, "CANCEL")){

		sipd_proxy_cancel(sipd, req, oreq, hkey, ohkey);
	}
        else {
		sipd_proxy(sipd, req, hkey);
	}
}


static void tp_recv(const tp_t *tp, const msg_t *msg, void *arg)
{
	sipd_t *sipd = (sipd_t *)arg;

	if(msg->response) {		
		if(sipd->stateless || (ctrans_resp(msg) != OK))
			sipd_proxy_sl_resp(sipd, msg);
	}
	else if(msg->request) {
		if((!sapi_strcmp(msg->request->method, "REGISTER") && sipd->stateless) || (strans_req(tp, msg, req, err, sipd) != OK))
			sipd_proxy_sl_req(sipd, msg);
	}
}


int main(int argc, char **argv)
{
	sipd_t *sipd = sipd_init();

	if(!sipd)
		return(-1);

	sapi_daemon();
	sapi_log_init("/var/log/sipd.log");
	sipd_proxy_init();
	sapi_oneshot_init();
	ctrans_init();
	strans_init();
	ldb_init();
	status_init(sipd->start_time);
	tp_init(sipd->url->port, tp_recv, sipd);
	
	if(sipd->stateless)
		SIPD_DEBUG_PRINTF("Server Ready [stateless]");
	else
		SIPD_DEBUG_PRINTF("Server Ready [stateful]");

	// @todo
	for(;;) {
		sleep(2000);
	}
	
	return(0);
}
