#include <stdlib.h> // getenv()
#include <time.h> // time()
#include <stdio.h> // fprintf()
#include <sapi.h>
#include <msg.h>
#include "sipd.h"


sipd_t *sipd_init(void)
{
	sipd_t *sipd = NULL;

	if(!getenv("SIPD_URI")) {
		fprintf(stderr, "Environment varaiable 'SIPD_URI' not set\n");
		goto err;
	}

	if(!getenv("SIPD_DOMAIN")) {
		fprintf(stderr, "Environment varaiable 'SIPD_DOMAIN' not set\n");
		goto err;
	}

	sipd = (sipd_t *)sapi_malloc((u32_t)sizeof(sipd_t));

	sipd->url = msg_url_alloc();

	if(msg_url_parse(sipd->url, getenv("SIPD_URI")) != OK) {
		fprintf(stderr, "Error parsing URI: %s\n", getenv("SIPD_URI"));
		goto err;
	}

	sipd->domain      = sapi_strdup(getenv("SIPD_DOMAIN"));
	sipd->start_time  = (u32_t)time(NULL);

	if(getenv("SIPD_STATELESS"))
		sipd->stateless = TRUE;

	return(sipd);
err:
	sipd_free(sipd);
	return(NULL);
}


void sipd_free(sipd_t *sipd)
{
	if(!sipd)
		return;

	msg_url_free(sipd->url);
	sapi_free(sipd->domain);

	sapi_free(sipd);
}
