#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include <strans.h>
#include <ldb.h>
#include <status.h>
#include <version.h>


#define TCP_LISTENQ 20
#define STATUS_PORT 6050

typedef struct {
	int fd;
} packet_t;


static fifo_t *__recvq = NULL;
pthread_t __tcp_listen;
pthread_t __tcp_read;
static u32_t __start = 0;


static packet_t *packet_alloc(void)
{
	return((packet_t *)sapi_malloc(sizeof(packet_t)));
}


static void packet_free(packet_t *packet)
{
	if(!packet)
		return;

	sapi_free(packet);
}


static u8_t *get_uptime(void)
{
    u32_t days, hrs, mins, secs;
    u32_t tsecs = time(NULL) - __start;
    u8_t *uptime = NULL;

    secs = tsecs % 60;
    tsecs /= 60;
    mins = tsecs % 60;
    tsecs /= 60;
    hrs = tsecs % 24;
    days = tsecs / 24;

    if(days)
	    sapi_saprintf(&uptime, " %ld day%s", days, days == 1 ? "" : "s");
    if(hrs)
	    sapi_saprintf(&uptime, " %ld hour%s", hrs, hrs == 1 ? "" : "s");
    if(mins)
	    sapi_saprintf(&uptime, " %ld minute%s", mins, mins == 1 ? "" : "s");
    if(secs)
	    sapi_saprintf(&uptime, " %ld second%s", secs, secs == 1 ? "" : "s");

    return(uptime);
}


static u8_t *status_html(void)
{
	u8_t *reply = NULL, *tmp;

	// sapi_logf("Handling Status Request");

	/* HTTP Response with headers */
	sapi_saprintf(&reply, "HTTP/1.0 200 OK\r\n");
	// sapi_saprintf(&reply, "Server: Apache/1.3.26 (Unix)\r\n");
	// sapi_saprintf(&reply, "Connection: close\r\n");
	sapi_saprintf(&reply, "Refresh: 5\r\n");
	sapi_saprintf(&reply, "Content-Type: text/html\r\n");
	sapi_saprintf(&reply, "\r\n");

	/* HTML Content */
	sapi_saprintf(&reply, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n");
	sapi_saprintf(&reply, "<html>\n");
	sapi_saprintf(&reply, "<head>\n");
	sapi_saprintf(&reply, "<title>SIP Server Status</title>\n");
	sapi_saprintf(&reply, "</head>\n");
	sapi_saprintf(&reply, "<body>\n");
	sapi_saprintf(&reply, "<h1>SIP Server Status</h1>\n\n");
	
	sapi_saprintf(&reply, "<pre>\n");

	sapi_saprintf(&reply, "Server Version      : " SIPD_VERSION "\n");
	sapi_saprintf(&reply, "Server Built        : " __DATE__ "\n");
	tmp = get_uptime();
	sapi_saprintf(&reply, "Server Uptime       :%s\n", tmp);
	sapi_free(tmp);
	sapi_saprintf(&reply, "Memory Allocations #: %i\n", sapi_nmalloced());

	sapi_saprintf(&reply, "</pre>\n\n");

	sapi_saprintf(&reply, "<hr>\n\n");

	/* Location */
	tmp = ldb_dump_html();
	sapi_saprintf(&reply, "%s", tmp);
	sapi_free(tmp);	

	sapi_saprintf(&reply, "<hr>\n\n");
	sapi_saprintf(&reply, "  <table bgcolor=\"#ffffff\" border=\"0\" cellpadding=\"7\" cellspacing=\"7\">\n");
	sapi_saprintf(&reply, " <tr>\n\n");
        sapi_saprintf(&reply, "  <td valign=\"top\">\n\n");


	/* Client Transactions */
	sapi_saprintf(&reply, "   <table bgcolor=\"#ffffff\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n");
	sapi_saprintf(&reply, "    <tr bgcolor=\"1478EB\">\n");
	sapi_saprintf(&reply, "     <td colspan=\"2\"><font face=\"Arial,Helvetica\" color=\"#ffffff\"><b>Client Transactions</b></font></td>\n");
	sapi_saprintf(&reply, "    </tr>\n");
	tmp = ctrans_status_str();
	sapi_saprintf(&reply, "%s", tmp);
	sapi_free(tmp);	
	sapi_saprintf(&reply, "   </table>\n");


	sapi_saprintf(&reply, "  </td>\n\n");
	sapi_saprintf(&reply, "  <td valign=\"top\">\n\n");


	/* Server Transactions */
	sapi_saprintf(&reply, "   <table bgcolor=\"#ffffff\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n");
	sapi_saprintf(&reply, "    <tr bgcolor=\"1478EB\">\n");
	sapi_saprintf(&reply, "     <td colspan=\"2\"><font face=\"Arial,Helvetica\" color=\"#ffffff\"><b>Server Transactions</b></font></td>\n");
	sapi_saprintf(&reply, "    </tr>\n");
	tmp = strans_status_str();
	sapi_saprintf(&reply, "%s", tmp);
	sapi_free(tmp);	
	sapi_saprintf(&reply, "   </table>\n");


	sapi_saprintf(&reply, "  </td>\n\n");
	sapi_saprintf(&reply, "  <td valign=\"top\">\n\n");


	/* Transport Receive Queue */
	sapi_saprintf(&reply, "   <table bgcolor=\"#ffffff\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n");
	sapi_saprintf(&reply, "    <tr bgcolor=\"1478EB\">\n");
	sapi_saprintf(&reply, "     <td colspan=\"2\"><font face=\"Arial,Helvetica\" color=\"#ffffff\"><b>Transport Receive Queue</b></font></td>\n");
	sapi_saprintf(&reply, "    </tr>\n");
	tmp = tp_status_str_recv();
	sapi_saprintf(&reply, "%s", tmp);
	sapi_free(tmp);	
	sapi_saprintf(&reply, "   </table>\n");


	sapi_saprintf(&reply, "  </td>\n\n");
	sapi_saprintf(&reply, "  <td valign=\"top\">\n\n");


	/* Transport Send Queue */
	sapi_saprintf(&reply, "   <table bgcolor=\"#ffffff\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n");
	sapi_saprintf(&reply, "    <tr bgcolor=\"1478EB\">\n");
	sapi_saprintf(&reply, "     <td colspan=\"2\"><font face=\"Arial,Helvetica\" color=\"#ffffff\"><b>Transport Send Queue</b></font></td>\n");
	sapi_saprintf(&reply, "    </tr>\n");
	tmp = tp_status_str_send();
	sapi_saprintf(&reply, "%s", tmp);
	sapi_free(tmp);	
	sapi_saprintf(&reply, "   </table>\n");


	sapi_saprintf(&reply, "  </td>\n\n");
	sapi_saprintf(&reply, "  <td valign=\"top\">\n\n");


	/* Logger */
	sapi_saprintf(&reply, "   <table bgcolor=\"#ffffff\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n");
	sapi_saprintf(&reply, "    <tr bgcolor=\"1478EB\">\n");
	sapi_saprintf(&reply, "     <td colspan=\"2\"><font face=\"Arial,Helvetica\" color=\"#ffffff\"><b>System Logger Status</b></font></td>\n");
	sapi_saprintf(&reply, "    </tr>\n");
	tmp = sapi_log_status_str();
	sapi_saprintf(&reply, "%s", tmp);
	sapi_free(tmp);	
	sapi_saprintf(&reply, "   </table>\n");


	sapi_saprintf(&reply, "  </td>\n");
	sapi_saprintf(&reply, " </tr>\n");
        sapi_saprintf(&reply, "</table>\n\n");

	sapi_saprintf(&reply, "<hr>\n\n");
	
	sapi_saprintf(&reply, "</body>\n");
	sapi_saprintf(&reply, "</html>\n");

	return(reply);
}


static void *tcp_listen_thread(void *arg)
{
	int fd = -1, on = 1;
	struct sockaddr_in sa;

	fd = socket(AF_INET, SOCK_STREAM, 0);
	if(fd == -1) {
		sapi_logf("tcp_listen(): socket(): %s", strerror(errno));
		goto err;
	}

	bzero(&(sa), sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = htonl(INADDR_ANY);
	sa.sin_port = htons((unsigned short int)STATUS_PORT);

	if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) {
		sapi_logf("tcp_listen(): setsockopt(): %s", strerror(errno));
		goto err;
	}

	if(bind(fd, (struct sockaddr *)&sa, (socklen_t)sizeof(sa)) < 0) {
		sapi_logf("tcp_listen(): bind(): %s", strerror(errno));
		goto err;
	}
	
	if(listen(fd, (int)TCP_LISTENQ) < 0) {
		sapi_logf("tcp_listen(): listen(): %s", strerror(errno));
		goto err;
	}

	while(__recvq) {
		packet_t *p = packet_alloc();
		
		p->fd = accept(fd, NULL, NULL);
		sapi_fifo_put(__recvq, p);

		// sapi_logf("tcp_listen(): got connection");
	}

 err:
	sapi_logf("tcp_listen(): exit");

	return(NULL);
}


static void *tcp_read_thread(void *arg)
{
	while(__recvq) {
		u8_t buf[64000];
		packet_t *p = sapi_fifo_get(__recvq);
		
		if(read(p->fd, buf, sizeof(buf)) > 0) {
			
			u8_t *str = status_html();
			write(p->fd, str, (size_t)sapi_strlen(str));
			sapi_free(str);

			// sapi_logf("tcp_read_thread(): processed request");
		}
		else {
			sapi_logf("tcp_read_thread(): Got empty packet; nmaped?");
		}

		close(p->fd);
		packet_free(p);
		// sapi_logf("tcp_read(): processed request");
	}

	return(NULL);
}


void status_init(u32_t start)
{
	__recvq = sapi_fifo_alloc();
	__start = start;
	
	pthread_create(&__tcp_listen, NULL, tcp_listen_thread, NULL);
	pthread_create(&__tcp_read, NULL, tcp_read_thread, NULL);

	sapi_logf("Status service is up and running");
}


void status_status(void)
{
	if(!__recvq)
		return;

	sapi_fifo_status(__recvq);
}
