#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <strans.h>
#include "st.h"


st_t *st_alloc(void)
{
        st_t *st = (st_t *)sapi_malloc((u32_t)sizeof(st_t));

        st->nrefs = 1;
        sapi_mutex_init(&st->mutex);

        return(st);
}


static void st_free(st_t *st)
{
	if(!st) {
		STRANS_DEBUG_PRINTF("st_free(): tried to free NULL pointer");
		return;
	}

	sapi_free(st->hkey);
	tp_deref(st->tp);
	msg_deref(st->req);
	msg_deref(st->resp);

	sapi_free(st);
}


st_t *st_ref(st_t *st)
{
        if(!st)
                return(NULL);

        sapi_mutex_lock(&st->mutex);
	st->nrefs++;
        sapi_mutex_unlock(&st->mutex);

        return(st);
}


void st_deref(void *arg)
{
	st_t *st = (st_t *)arg;
	u32_t nrefs;

        if(!st)
                return;

        sapi_mutex_lock(&st->mutex);
        nrefs = --st->nrefs;
        sapi_mutex_unlock(&st->mutex);

        if(!nrefs)
                st_free(st);
}


void st_set_state(st_t *st, state_t state)
{
        if(!st)
                return;

        sapi_mutex_lock(&st->mutex);
	st->state = state;
        sapi_mutex_unlock(&st->mutex);
}


state_t st_get_state(st_t *st)
{
	state_t state;

        if(!st)
                return(INVALID);

        sapi_mutex_lock(&st->mutex);
	state = st->state;
        sapi_mutex_unlock(&st->mutex);

	return(state);
}


void st_inc_ntx(st_t *st)
{
        if(!st)
                return;

        sapi_mutex_lock(&st->mutex);
	st->ntx++;
        sapi_mutex_unlock(&st->mutex);
}


u32_t st_get_ntx(st_t *st)
{
	u32_t ntx;

        if(!st)
                return(0);

        sapi_mutex_lock(&st->mutex);
	ntx = st->ntx;
        sapi_mutex_unlock(&st->mutex);

	return(ntx);
}
