#include <unistd.h> // for write()
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include <strans.h>
#include <rtp.h>
#include <dsp.h>
#include <g711.h>
#include "ua.h"


static void err(const u8_t *reason, void *arg)
{
	sapi_logf("Server Transaction Error : %s", reason);
}


static void req(const msg_t *req, const msg_t *oreq, const u8_t *hkey, const u8_t *ohkey, void *arg)
{
	msg_t *resp;

	if(sapi_strcmp(req->request->method, "INVITE")) {

		u8_t *from_str;
		address_t *redir;

		from_str = msg_address_display_str(req->from);
		sapi_printf("Incoming call from '%s'", from_str);
		sapi_free(from_str);
		sapi_fprintf(1, "sip> ");

		resp = msg_response(req, "200", "OK");
		
		redir = msg_address_alloc();

		if(msg_address_parse(redir, "sip:einar@sxdesign.com") == OK)
			msg_add_contact(resp, redir);

		msg_address_free(redir);		

		strans_resp(hkey, resp);

		msg_deref(resp);

	}
	else if(sapi_strcmp(req->request->method, "CANCEL")){

		if(ohkey && oreq) {

			resp = msg_response(req, "200", "OK");
			strans_resp(hkey, resp);
			msg_deref(resp);
			
			resp = msg_response(oreq, "487", "Request Terminated");			
			strans_resp(ohkey, resp);
			msg_deref(resp);
		}
		else {
			resp = msg_response(req, "481", "Transaction Does Not Exist");		
			strans_resp(hkey, resp);
			msg_deref(resp);
		}

		sapi_printf("Request canceled");
	}
	else if(sapi_strcmp(req->request->method, "MESSAGE")) {
		
		resp = msg_response(req, "200", "Okay");
		strans_resp(hkey, resp);
		msg_deref(resp);
		
		ua_message_recv(req);
	}
	else {
		sapi_printf("Got %s request", req->request->method);
		
		resp = msg_response(req, "200", "OK");
		strans_resp(hkey, resp);
		msg_deref(resp);
	}
}


static void tp_recv(const tp_t *tp, const msg_t *msg, void *param)
{
	ua_t *ua = (ua_t *)param;

	if(msg->response) {		
		ctrans_resp(msg);
	}
	else if(msg->request) {
		strans_req(tp, msg, req, err, ua);
	}
}


static void autoregister(void *arg, u8_t *sarg)
{
	ua_t *ua = (ua_t *)arg;

	ua_register(ua, 360);
	sapi_oneshot(3000, autoregister, ua, NULL);
}


static void input_recv(u8_t *cmd, ua_t *ua)
{
	if(!sapi_strlen(cmd)) {
		;
	}
	else if(sapi_strcmp(cmd, "m")) {

		address_t *addr = ua_input_address(ua);
		
		if(addr) {
			ua_message_send(ua, addr);
			msg_address_free(addr);
		}
	}
	else if(sapi_strcmp(cmd, "i")) {
		
		address_t *addr = ua_input_address(ua);
		
		if(addr) {
			ua_invite(ua, addr);
			msg_address_free(addr);
		}
	}
	else if(sapi_strcmp(cmd, "t")) {

		address_t *addr = ua_input_address(ua);
		
		if(addr) {
			ua_test(ua, addr, "ACK");
			msg_address_free(addr);
		}
	}
	else if(sapi_strcmp(cmd, "b")) {

		address_t *addr = ua_input_address(ua);
		
		if(addr) {
			ua_bye(ua, addr);
			msg_address_free(addr);
		}
	}
	else if(sapi_strcmp(cmd, "c")) {

		ua_cancel(ua);
	}
	else if(sapi_strcmp(cmd, "dsp")) {

		dsp_status();
	}
	else if(sapi_strcmp(cmd, "rtp")) {

		rtp_status();
	}
	else if(sapi_strcmp(cmd, "rec")) {

		int fd = open("/tmp/test.wav", O_RDONLY), n;
		u8_t buf[4000];

		if(fd < 0) {
			sapi_printf("/tmp/test.wav: %s", strerror(errno));
		}
		else {
			sapi_printf("Recording...");
			
			while((n = read(fd, buf, sizeof(buf)))) {

				rtp_send("193.71.98.243", 5004, (void *)buf, n, 0);
				usleep(10000);
			}

			close(fd);
		}
	}
	else if(sapi_strcmp(cmd, "s")) {

		sapi_printf("-------------------------------------------");
		sapi_printf("Memory Alloation#: %i", sapi_nmalloced());
		sapi_oneshot_status();
		sapi_printf("Transport Status:");
		tp_status();
		sapi_printf("Logger Status:");
		sapi_log_status();
		sapi_printf("Client transaction Status:");
		ctrans_status();
		sapi_printf("Server transaction Status:");
		strans_status();
	}
	else {
		sapi_printf("???? %s", cmd);
	}

	sapi_fprintf(1, "sip> ");
}


void rtp_recv(void *payload, u32_t payload_size, void *arg)
{
	s16_t pcm_buf[payload_size];
	u16_t i;
	u8_t *encoded = (u8_t *)payload;

	for(i=0;i<payload_size;i++)
		pcm_buf[i] = alaw2pcm(*encoded++);

	sapi_logf("Got RTP packet of %u bytes", payload_size);

	dsp_write(pcm_buf, payload_size);
}


void dsp_recv(void *payload, size_t payload_size, void *arg)
{
	//sapi_logf("Got DSP packet of %u bytes", payload_size);

	//	rtp_send("127.0.0.1", 5004, payload, payload_size);
}


int main(int argc, char **argv)
{
	ua_t *ua;
	u8_t buf[64000];
	int rc;

	sapi_log_init("/var/log/ua.log");
	sapi_oneshot_init();

	ua = ua_init();
	if(!ua)
		return(-1);
	
	tp_init(ua->local->url->port, tp_recv, ua);
	ctrans_init();
	strans_init();

	rtp_init(RTP_PORT, rtp_recv, ua);
	dsp_init(dsp_recv, ua);

	autoregister(ua, NULL);
	
	sapi_logf("UA Ready");

	sapi_fprintf(1, "sip> ");
	
	for(;;) {
		rc = read(0, buf, sizeof(buf) - 1);
		if(rc > 0) {
			buf[rc-1] = '\0';
			input_recv(buf, ua);
		}
		else if(rc == 0)
			break;
	}

	sapi_printf("Unregistering...");
	ua_register(ua, 0);
	sleep(1);

	ua_free(ua);

	sapi_printf("Nmalloced:   %i", sapi_nmalloced());

	return(0);
}
