#include <stdlib.h>
#include <unistd.h>
#include <version.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include "ua.h"


static void err(const msg_t *req, const u8_t *reason, void *arg, u8_t *sarg)
{
	sapi_printf("Message Failed: %s", reason);
	sapi_fprintf(1, "sip> ");
}

static void resp(const msg_t *req, const msg_t *resp, void *arg, u8_t *sarg)
{
#if 1
	sapi_printf("Message Response: %s (%s)", resp->response->statuscode, 
		    resp->response->reasonphrase);
	sapi_fprintf(1, "sip> ");
#endif
}


u8_t *input_message(void)
{
	u8_t buf[8000], *input = NULL;
	int rc;

	sapi_printf("Enter Message:");

	for(;;) {
		rc = read(0, buf, sizeof(buf) - 1);
		if(rc > 0) {
			buf[rc] = '\0';
			sapi_saprintf(&input, buf);
		}
		else if(rc == 0)
			break;
	}

	return(input);
}


void ua_message_send(ua_t *ua, const address_t *addr)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(ua->local->url->host, ua->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), ua->global->url->host);
	tp_t *tp        = tp_alloc();
	u8_t *input     = input_message();
	
	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	msg_via_addparam(via, "branch", branch);

	if(input) {
		
		msg_set_request(msg, "MESSAGE", addr->url);
		msg_add_via(msg, via);
		msg_set_to(msg, addr);
		msg_set_from(msg, ua->global);
		msg_set_call_id(msg, call_id);
		msg_set_cseq(msg, ua->cseq++, "MESSAGE");
		msg_add_header(msg, "Content-Type", "text/plain");
		msg_add_header_u32(msg, "Content-Length", sapi_strlen(input));
		
		msg_set_body(msg, input);

		ctrans_req(tp, msg, resp, err, NULL, NULL);
	}

	sapi_free(input);
	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}


void ua_message_recv(const msg_t *req)
{
	u8_t *from_str;

	from_str = msg_address_display_str(req->from);
	sapi_printf("Incoming message from '%s':\n\a", from_str);
	sapi_free(from_str);
	
	if(req->body) {
		sapi_printf(req->body);
	}
	else {
		sapi_printf("<empty body>");
	}
	
	sapi_fprintf(1, "sip> ");
}
