#include <stdlib.h>
#include <unistd.h>
#include <version.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include "ua.h"


static void err(const msg_t *req, const u8_t *reason, void *arg, u8_t *sarg)
{
	sapi_printf("Message Failed: %s", reason);
	sapi_fprintf(1, "sip> ");
}

static void resp(const msg_t *req, const msg_t *resp, void *arg, u8_t *sarg)
{
	sapi_printf("Message Response: %s (%s)", resp->response->statuscode, 
		    resp->response->reasonphrase);
	sapi_fprintf(1, "sip> ");
}


void ua_test(ua_t *ua, const address_t *addr, const u8_t *method)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(ua->local->url->host, ua->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), ua->global->url->host);
	tp_t *tp        = tp_alloc();
	
	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	msg_via_addparam(via, "branch", branch);

	msg_set_request(msg, method, addr->url);
	msg_add_via(msg, via);
	msg_set_to(msg, addr);
	msg_set_from(msg, ua->global);
	msg_set_call_id(msg, call_id);
	msg_set_cseq(msg, ua->cseq++, method);

	ctrans_req(tp, msg, resp, err, NULL, NULL);

	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}
