#include <stdlib.h> // for rand()
#include <sys/types.h> // for open()
#include <sys/stat.h> // for open()
#include <fcntl.h> // for open()
#include <string.h> // for strerror()
#include <errno.h> // for strerror()
#include <unistd.h>
#include <time.h>
#include <version.h>
#include <sapi.h>
#include <msg.h>
#include <tp.h>
#include <ctrans.h>
#include <sdp.h>
#include "ua.h"
#include <ctype.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netdb.h>              /* gethostbyname, getnetbyname */
#include <net/ethernet.h>       /* struct ether_addr */
#include <linux/wireless.h>


#define DOMAINNAME "sxdesign.com"
#define PROXY      "sip:sxdesign.com"


ua_t *ua_init(void)
{
	ua_t *ua = (ua_t *)sapi_malloc((u32_t)sizeof(ua_t));
	u8_t hostname[255];

	gethostname(hostname, sizeof(hostname));
	
	ua->str_local  = sapi_sprintf("sip:%s@%s.%s:%u", getlogin(), hostname, DOMAINNAME, 1024 + (u16_t)getpid());
	ua->str_global = sapi_sprintf("sip:%s@%s", getlogin(), DOMAINNAME);
	ua->str_proxy  = sapi_strdup(PROXY);

	ua->local  = msg_address_alloc();
	ua->global = msg_address_alloc();
	ua->proxy  = msg_url_alloc();

	if(msg_address_parse(ua->local, ua->str_local) != OK) {
		sapi_printf("Error parsing local address: '%s'", ua->str_local);
		goto out_err;
	}

	if(msg_address_parse(ua->global, ua->str_global) != OK) {
		sapi_printf("Error parsing global address: '%s'", ua->str_global);
		goto out_err;
	}

	if(msg_url_parse(ua->proxy, ua->str_proxy) != OK) {
		sapi_printf("Error parsing proxy URI: '%s'", ua->str_proxy);
		goto out_err;
	}	

	sapi_printf("SIP UA Version: " SIPD_VERSION);
	sapi_printf("Local Address : '%s'", ua->str_local);
	sapi_printf("Global Address: '%s'", ua->str_global);
	sapi_printf("Proxy URI     : '%s'", ua->str_proxy);

	return(ua);

out_err:
	ua_free(ua);
	return(NULL);
}


void ua_free(ua_t *ua)
{
	if(!ua)
		return;

	sapi_free(ua->str_local);
	sapi_free(ua->str_global);
	sapi_free(ua->str_proxy);

	msg_address_free(ua->local);
	msg_address_free(ua->global);
	msg_url_free(ua->proxy);

	msg_address_free(ua->last);

	msg_deref(ua->inv);

	sapi_free(ua);
}


static void err(const msg_t *req, const u8_t *reason, void *arg, u8_t *sarg)
{
#if 1
	sapi_printf("%s Error: %s", req->request->method, reason);

	sapi_fprintf(1, "sip> ");
#endif
}

static void resp(const msg_t *req, const msg_t *resp, void *arg, u8_t *sarg)
{
#if 1
	sapi_printf("%s Response: %s (%s)",
		  req->request->method,
		  resp->response->statuscode,
		  resp->response->reasonphrase);

	sapi_fprintf(1, "sip> ");
#endif
}


char *get_ssid(void)
{
	struct iwreq wrq;
	char *essid = sapi_malloc(IW_ESSID_MAX_SIZE + 1);
	int ret;

	static const int families[] = {
		AF_INET, AF_IPX, AF_AX25, AF_APPLETALK
	};
	unsigned int  i;
	int  sock;
        
	/* Try all families we support */
        for(i = 0; i < sizeof(families)/sizeof(int); ++i) {
		/* Try to open the socket, if success returns it */
		sock = socket(families[i], SOCK_DGRAM, 0);
		if(sock >= 0)
		        break;
	}

	wrq.u.essid.pointer = (caddr_t) essid;
	wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;

	strncpy(wrq.ifr_name, "eth1", IFNAMSIZ); /** @todo find interface */

	ret = ioctl(sock, SIOCGIWESSID, &wrq);
	if(ret < 0 || strlen(essid) <= 0) {
		sapi_free(essid);
		essid = NULL;
	}

	close(sock);
	return(essid);
}


void ua_register(ua_t *ua, s32_t expires)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(ua->local->url->host, ua->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), ua->global->url->host);
	tp_t *tp        = tp_alloc();
	static u32_t cseq;
	address_t *contact;
	char *ssid = get_ssid();
	
	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	msg_via_addparam(via, "branch", branch);

	contact = msg_address_dup(ua->local);
	msg_address_addparam_u32(contact, "expires", expires);

	msg_set_request(msg, "REGISTER", ua->proxy);
	msg_add_via(msg, via);
	msg_set_to(msg, ua->global);
	msg_set_from(msg, ua->global);
	msg_add_contact(msg, contact);
	msg_set_call_id(msg, call_id);
	msg_set_cseq(msg, cseq++, "REGISTER");
	msg_set_user_agent(msg, USER_AGENT);
	if(ssid) {
		sapi_logf("ssid '%s'\n", ssid);
		msg_add_header(msg, "X-SSID", ssid);
		free(ssid);
	}
	
	// ctrans_req(tp, msg, resp, err, NULL, NULL);
	ctrans_req(tp, msg, NULL, NULL, NULL, NULL);

	msg_address_free(contact);
	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}


void ua_invite(ua_t *ua, const address_t *addr)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(ua->local->url->host, ua->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), ua->global->url->host);
	tp_t *tp        = tp_alloc();
	static u32_t cseq;
	u8_t *body;
	sdp_t *sdp = sdp_create("127.0.0.1", "audio", RTP_PORT, "RTP/AVP");
	if(!sdp) {
		sapi_printf("could not create SDP object");
		return;
	}
	sdp_media_add(sdp, 0, "PCMU", 8000, NULL);
	body = sdp_str(sdp);
	if(!body) {
		sdp_free(sdp);
		sapi_printf("could not create SDP body");
		return;
	}
	sdp_free(sdp);

	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	if(addr) {
		
		msg_via_addparam(via, "branch", branch);
		msg_via_addparam(via, "sent-by", ua->local->url->host);
		
		msg_set_request(msg, "INVITE", addr->url);
		msg_add_via(msg, via);
		msg_set_to(msg, addr);
		msg_set_from(msg, ua->global);
		msg_add_contact(msg, ua->local);
		msg_add_contact(msg, ua->local);
		msg_set_call_id(msg, call_id);
		msg_set_cseq(msg, cseq++, "INVITE");
		msg_set_max_forwards(msg, 70);
		msg_add_header(msg, "User-Agent", "sipd");
		
		msg_add_header(msg, "Content-Type", "application/sdp");
		msg_add_header_u32(msg, "Content-Length", sapi_strlen(body));
		msg_set_body(msg, body);

		msg_address_addparam(msg->to, "tag", "tt");
		msg_address_addparam(msg->from, "tag", "ft");

		ctrans_req(tp, msg, resp, err, NULL, NULL);

		if(ua->inv)
			msg_deref(ua->inv);

		ua->inv = msg_ref(msg);
	}

	sapi_free(body);
	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}


/**
 * Send BYE request.
 *
 * @todo This implementation is not correct - fix it.
 */
void ua_bye(ua_t *ua, const address_t *addr)
{
	msg_t *msg      = msg_alloc();
	u8_t *branch    = sapi_sprintf("z9hG4bK%i%i", time(NULL), rand());
	via_t *via      = msg_via_create(ua->local->url->host, ua->local->url->port);
	u8_t *call_id   = sapi_sprintf("%i%i@%s", time(NULL), rand(), ua->global->url->host);
	tp_t *tp        = tp_alloc();
	static u32_t cseq;

	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	if(addr) {
		
		msg_via_addparam(via, "branch", branch);
		msg_via_addparam(via, "sent-by", ua->local->url->host);
		
		msg_set_request(msg, "BYE", addr->url);
		msg_add_via(msg, via);
		msg_set_to(msg, addr);
		msg_set_from(msg, ua->global);
		msg_add_contact(msg, ua->local);
		msg_add_contact(msg, ua->local);
		msg_set_call_id(msg, call_id);
		msg_set_cseq(msg, cseq++, "BYE");
		msg_set_max_forwards(msg, 70);
		
		msg_address_addparam(msg->to, "tag", "tt");
		msg_address_addparam(msg->from, "tag", "ft");

		ctrans_req(tp, msg, resp, err, NULL, NULL);

		if(ua->inv)
			msg_deref(ua->inv);

		ua->inv = msg_ref(msg);
	}
	
	tp_deref(tp);
	sapi_free(call_id);
	msg_via_free(via);
	sapi_free(branch);
	msg_deref(msg);
}


void ua_cancel(ua_t *ua)
{
	msg_t *msg;
	tp_t *tp;

	if(!ua->inv) {
		sapi_printf("No Invite to Cancel");
		return;
	}

	tp  = tp_alloc();

	tp->host  = sapi_strdup(ua->proxy->host);
	tp->port  = ua->proxy->port;
	tp->proto = UDP;

	msg = msg_cancel(ua->inv);

	ctrans_req(tp, msg, resp, err, NULL, NULL);

	msg_deref(msg);
	tp_deref(tp);

	msg_deref(ua->inv);
	ua->inv = NULL;
}


address_t *ua_input_address(ua_t *ua)
{
	u8_t buf[8000] = "";
	int rc;
	
	if(!ua->last) {
		sapi_printf2("Enter SIP address: ");
	}
	else {
		u8_t *addr_str = msg_address_str(ua->last);

		sapi_printf2("Enter SIP address (default: %s): ", addr_str);
		sapi_free(addr_str);
	}

	rc = read(0, buf, sizeof(buf) - 1);
	if(rc > 1) {
		buf[rc - 1] = '\0';

		if(ua->last)
			msg_address_free(ua->last);

		ua->last = msg_address_alloc();

		if(msg_address_parse(ua->last, buf) != OK) {
			sapi_printf("Malformed address: '%s'", buf);
			msg_address_free(ua->last);
			ua->last = NULL;
		}
	}

	return(msg_address_dup(ua->last));
}
